@@ -17,6 +17,7 @@
 import sys
 from logreduce.utils import files_iterator
 from logreduce.process import Classifier
+from logreduce.tokenizer import filename2modelname
 
 try:
     path = sys.argv[1]
@@ -24,9 +25,9 @@ except IndexError:
     print("usage: %s dir" % sys.argv[0])
     exit(1)
 
-groups = {}
+groups = {}  # type: ignore
 for filename, filename_rel in files_iterator(path):
-    bag_name = Classifier.filename2modelname(filename_rel)
+    bag_name = filename2modelname(filename_rel)
     groups.setdefault(bag_name, []).append(filename)
 
 for group_name, files in sorted(groups.items()):
